#include "../include/sv.h"
#include <string.h>
#include <stdbool.h>

static bool is_space(int c) {
    return c == ' ' || c == '\t' || c == '\n' || c == '\r' || c == '\v' || c == '\f';
}
static int to_lower(char c) {
  return (c >= 'A' && c <= 'Z') ? c + 'a' - 'A' : c;
}

SV sv_str(const char *s){
  SV str = {s == NULL ? 0 : strlen(s), s};
  return str;
}
SV sv_strn(const char *s, size_t len){
  SV str = {len, s};
  return str; 
}
SV sv_trim(SV s){
    while (s.len > 0 && is_space((int) *s.buf)) s.buf++, s.len--;
    while (s.len > 0 && is_space((int) *(s.buf + s.len - 1))) s.len--;
    return s;
}
SV sv_trim_left(SV s){
    while (s.len > 0 && is_space((int) *s.buf)) s.buf++, s.len--;
    return s;
}
SV sv_trim_right(SV s){
    while (s.len > 0 && is_space((int) *(s.buf + s.len - 1))) s.len--;
    return s;
}
int sv_cmp(const SV s1, const SV s2){
    size_t i = 0;
    while (i < s1.len && i < s2.len) {
        int c1 = s1.buf[i];
        int c2 = s2.buf[i];
        if (c1 < c2) return 0;
        if (c1 > c2) return 1;
        i++;
    }
    if (i < s1.len) return 1;
    if (i < s2.len) return 0;
    return 0;
}
int sv_casecmp(const SV s1, const SV s2) {
  size_t i = 0;
  while (i < s1.len && i < s2.len){
    int c1 = to_lower(s1.buf[i]);
    int c2 = to_lower(s2.buf[i]);
    if (c1 < c2) return -1;
    if (c1 > c2) return 1;
    i++;
  }
  if (i < s1.len) return 1;
  if (i < s2.len) return -1;
  return 0;
}
SV before_delim(const SV s, char *delims) {
    // Handle NULL pointer or empty string view
    if (s.buf == NULL || delims == NULL) {
        return sv_strn(NULL, 0);
    }

    size_t i = 0;
    while (i < s.len) {
        // Check if the current character is a delimiter
        for (size_t j = 0; delims[j] != '\0'; j++) {
            if (s.buf[i] == delims[j]) {
                SV result = sv_strn(s.buf, i);
                return result;
            }
        }
        i++;
    }

    // Return the whole string if no delimiter was found
    SV result = sv_strn(s.buf, i);
    return result;
}
SV after_delim(const SV s, char *delims) {
    // Handle NULL pointer or empty string view
    if (s.buf == NULL || delims == NULL) {
        return sv_strn(NULL, 0);
    }

    size_t i = 0;
    while (i < s.len) {
        // Check if the current character is a delimiter
        for (size_t j = 0; delims[j] != '\0'; j++) {
            if (s.buf[i] == delims[j]) {
                SV result = sv_strn(s.buf+i+1, s.len-i-1);
                return result;
            }
        }
        i++;
    }

    // Return the whole string if no delimiter was found
    SV result = sv_strn(s.buf, i);
    return result;
}
SV sv_strstr(const SV haystack, const SV needle){
    size_t i;
    if (needle.len > haystack.len) return sv_strn(NULL,0);
    if (needle.len == 0) return haystack;
    for (i = 0; i <= haystack.len - needle.len; i++) {
        if (memcmp(haystack.buf + i, needle.buf, needle.len) == 0) {
            return sv_strn(haystack.buf + i, haystack.len -i);
        }
    }
    return sv_strn(NULL,0);
}
